from .schema_builder import geom, point
from .trx import insert_point, update, delete, get_container


all = (
    'geom',
    'point',
    'insert_point',
    'update',
    'delete',
    'get_container',
)



from typing import List, Tuple

import wfs_trx.schema_builder as builder



def insert_point(*,
    namespaces: List[dict],
    featureType: str,
    props: List[dict],
    point: Tuple[float, float],
    srs: int = 3857
) -> str:
    return builder.trx(
        namespaces,
        builder.insert(
            featureType,
            f"""
            {builder.props(props)}
            {builder.geom(builder.point(point, srs))}
            """))


def update(*,
    fid: str,
    namespaces: List[dict],
    typeName: str,
    props: List[dict]
) -> str:
    return builder.trx(
        namespaces,
        builder.update(typeName,
            f"""{' '.join([builder.property(prop['key'], prop['val']) for prop in props])}{builder.filter(fid)}"""))


def delete(*,
    fid: str,
    namespaces: List[dict],
    typeName: str
) -> str:
    return builder.trx(
        namespaces,
        builder.delete(typeName,
            builder.filter(fid)))
            

def get_container(
    namespaces: List[dict],
    typeName: str,
    geom_prop: str,
    point: Tuple[float, float],  
):
    return builder.get_feature(
        namespaces=namespaces,
        value=builder.query(
            typeName=typeName,
            value=builder.filter(
                value=builder.contains(
                    geom_prop=geom_prop,
                    geom=builder.point(coordinates=point, srs=3857)
                )
            )
        )
    )


from typing import List, Tuple, Union



def trx(
    namespaces: List[dict],
    value: str
) -> str:
    return f"""<wfs:Transaction service="WFS" version="1.0.0" xmlns:wfs="http://www.opengis.net/wfs" xmlns:ogc="http://www.opengis.net/ogc" xmlns:gml="http://www.opengis.net/gml" {' '.join(['%s="%s"' % (ns['key'], ns['val']) for ns in namespaces])}>{value}</wfs:Transaction>"""

def get_feature(
    namespaces: List[dict],
    value: str,
    output_format: str = "JSON"
):
    return f"""<wfs:GetFeature service="WFS" version="1.0.0" xmlns:wfs="http://www.opengis.net/wfs" xmlns:ogc="http://www.opengis.net/ogc" xmlns:gml="http://www.opengis.net/gml" {' '.join(['%s="%s"' % (ns['key'], ns['val']) for ns in namespaces])} outputFormat="{output_format}">{value}</wfs:GetFeature>"""

def insert(
    feature_type: str,
    value: str
) -> str:
    return f"<wfs:Insert><{feature_type}>{value}</{feature_type}></wfs:Insert>"


def props(
    props: List[dict]
) -> str:
    return f"{' '.join(['<%s>%s</%s>' % (prop['key'], prop['val'], prop['key']) for prop in props])}"


def geom(value: str) -> str:
    return f"<gml:geom>{value}</gml:geom>"


def point(
    coordinates: Tuple[float, float],
    srs: int
) -> str:
    return f"""<gml:Point srsName="http://www.opengis.net/gml/srs/epsg.xml#{srs}"><gml:coordinates xmlns:gml="http://www.opengis.net/gml" decimal="." cs="," ts=" ">{str(coordinates[0])},{str(coordinates[1])}</gml:coordinates></gml:Point>"""

def update(
    type_name: str,
    value: str
) -> str:
    return f"""<wfs:Update typeName="{type_name}">{value}</wfs:Update>"""


def filter(value: str) -> str:
    return f"""<Filter>{value}</Filter>"""


def feature_id(fid: str):
    return f"""<FeatureId fid="{fid}"/>"""


def property(name: str, value: Union[str, int]) -> str:
    return f"""<wfs:Property><wfs:Name>{name}</wfs:Name><wfs:Value>{value}</wfs:Value></wfs:Property>"""


def delete(
    type_name: str,
    value: str
) -> str:
    return f"""<wfs:Delete typeName="{type_name}">{value}</wfs:Delete>"""


def query(
    typeName: str,
    value: str
) -> str: 
    return f"""<wfs:Query typeName="{typeName}">{value}</wfs:Query>"""


def contains(
    geom_prop: str,
    geom: str
) -> str:
    return f"""<Contains><PropertyName>{geom_prop}</PropertyName>{geom}</Contains>"""


author_info = (("Mehdi Kamani", "itsmehdi97@gmail.com"),)

package_info = "generate xml request body for wfs write transactions."
package_license = "MIT License"

team_email = "itsmehdi97@gmail.com"

version_info = (0, 0, 3)

__author__ = ", ".join("{} <{}>".format(*info) for info in author_info)
__version__ = ".".join(map(str, version_info))